#include <Stepper.h> // Programmbibliothek
const int SPDrehung = 400; //Schritte pro Umdrehung, maximum bei 400
Stepper stepperWaagerecht(SPDrehung, 6, 7);// Schritte pro Umdrehung, Step Pin und Direction Pin festlegen
Stepper stepperSenkrecht (SPDrehung, 4, 5);
Stepper stepperRotation (SPDrehung, 2, 3);
//Trig- und Echo Pin festlegen für den Ultraschallsensor
int TRIG_PIN = 13; 
int ECHO_PIN = 12;

void setup() {

  float durchschnitt_x;// Variable,gibt durchschnittliche Entfernung zum Objekt an, später umgerechnet in Breite
  float durchschnitt_z;// Variable, gibt durchschnittliche Entfernung zum Objekt an, später umgerechnet in Länge
  float yWert; //Variable, später umgerechnet in Höhe

  pinMode(TRIG_PIN, OUTPUT); //Trigger Pin ist ausgang
  pinMode(ECHO_PIN, INPUT);// Echo pin ist eingang
  Serial.begin(9600); //9600 bps, serielle Komunikation

  stepperWaagerecht.setSpeed(250); // Geschwindigkeit in Umdrehungen/Minute
  durchschnitt_x = motorWaagerecht(-196); //Funktion zur Motorbewegung aufrufen, Richtung und Anzahl der Schritte festlegen

  // Die Kreisplatte wird bewegt, damit später Messwerte für Z-Koordinate aufgenommen werden können
  stepperRotation.setSpeed(200);
  // Kreisfläche soll sich um 90 Grad ( = -2400 Schritte) im Uhrzeigersinn drehen(, weil es sonst wegen dem Riemen hackt)
  stepperRotation.step(-2400);

  durchschnitt_z = motorWaagerecht(196);//Funktion zur Motorbewegung aufrufen, Richtung und Anzahl der Schritte festlegen

  // in die Mitte bewegen
  stepperWaagerecht.setSpeed(100);
  stepperWaagerecht.step(-3200); // Anzahl der Schritte die es zur Mitte braucht, bei 400 Schritte wären es 8 Umdrehungen zur Mitte

  
  int counterY = 0;

  float entfernung = 0;
//Sensor nach Oben bewegen
  while (entfernung <= 20) { // While Schleife
    stepperSenkrecht.step(+10); // nur ein Schritt nach oben
    stepperSenkrecht.setSpeed(100);

    //Messung aufnehmen
    entfernung = messung();


    // Wenn Entfernung größer als 20, dann ist das Objekt zuende, keine weitere Messaufnahme
    // Pro Motorschritt ein Millimeter, wird aufsummiert und ergibt ungefähr die Höhe
    if (entfernung <= 20) {
      counterY++;

    }
  }

  yWert = counterY;// neuer Variablenname für mehr Übersicht

  //Umrechnen der Werte:
  float breiteX = (17.5 - durchschnitt_x) * 2;
  float laengeZ = (17.5 - durchschnitt_z) * 2;
  float hoeheY =  (yWert / 10.0); // Umrechnung in cm
  //Werte werden an seriellen Port übergeben und durch Komma getrennt, am Ende ein Semikolon
  Serial.print(breiteX);
  Serial.print(",");
  Serial.print(hoeheY);
  Serial.print(",");
  Serial.print(laengeZ);
  Serial.print(";");

  //Sensor fährt in die Ausgangsposition, Position sollte vor Start überprüft und evtl. manuell korrigiert werden
  stepperSenkrecht.step(-2000); // Anzahl der Schritte die der Sensor maximal nach unten braucht
  stepperWaagerecht.step(3200); // Anzahl der Schritte die es zurück braucht, nach links 
}


void loop () {} //nicht benötigte Funktion, der Roboter soll nicht unendlich lange messen

//Funktion zur Messung der Entfernung, und Umrechnung dieser in Zentimeter
float messung () {

  digitalWrite(TRIG_PIN, LOW);//Spannung nimmt kurz ab beim senden d. Trigger-Signals -> rauschfreies Signal
  delayMicroseconds(2);
  digitalWrite(TRIG_PIN, HIGH);//Utraschallwelle schicken
  delayMicroseconds(10);       //für 10 Mikrosekunden pausieren
  digitalWrite(TRIG_PIN, LOW); // Schall abschalten
  long dauer = pulseIn(ECHO_PIN, HIGH); //zählt die Zeit, wie lange der Ultraschall braucht, bis er wieder zurückkommt
  float entfernung;
  entfernung = dauer / 29.00 / 2.00; //Entfernung in Zentimetern: Zeit/2 (Weil man ja nur eine Strecke berechnen möchte
  //und nicht die Strecke hin- und zurück). Den Wert multipliziert man mit der Schallgeschwindigkeit in Zentimeter/
  //Mikrosekunde (= 1/29)und erhält Wert in Zentimetern(hier wird durch 29 dividiert, was jedoch dasselbe ist).

  return entfernung; //Rückgabewert ist die Entfernung in Zentimeter
}

//Funktion um Sensor nach links und rechts zu fahren, Entfernung zu messen, Durchschnitt der Entfernung zu berechnen
//Rückgabewert ist dieser Durchschnitt
float motorWaagerecht(int schritte) {
  float summe = 0;
  int counter = 0;

  //werte für x und z
  for ( int x = 0; x <= 1536; x = x + 30) { //sensor bewegt sich nach rechts

    //Bewegen
    stepperWaagerecht.step(schritte / 2); //-196

    //Messung aufnehmen, entsprechende Funktion aufrufen
    float entfernung = messung();
    // If-Bedingung um nur Werte zu addieren, die kleiner als 20 und größer als 0 cm betragen,
    //also nur Werte die relevant für das Objekt sind.
    if (entfernung <= 20 && entfernung > 0) {

      summe = summe + entfernung; //Entfernungen summieren
      counter++; // zählt die durchläufe
    }
  }

  float durchschnitt = summe / counter;// Durchschnitt berechnen
  return durchschnitt;// Durchschnitt ist Rückgabewert

}
